@@ -10,57 +10,36 @@ lmest <- function(responsesFormula = NULL, latentFormula = NULL,
                                  al = NULL, be = NULL, si = NULL,
                                  rho = NULL, la = NULL, PI = NULL,
                                  fixPsi = FALSE),
-                  fort = TRUE, seed = NULL)
-{
-  if(inherits(data, "lmestData"))
-  {
+                  fort = TRUE, seed = NULL, ntry = 0){
+
+  if(inherits(data, "lmestData")){
     data <- data$data
-  }else if(!is.data.frame(data))
-  {
+  }else if(!is.data.frame(data)){
     data <- as.data.frame(data)
     stop("A data.frame must be provided")
   }
 
-  if(start == 2)
-  {
-    if(is.null(parInit))
-    {
+  if(start == 2){
+    if(is.null(parInit)){
       stop("With start = 2, initial parameters must be provided")
-
-    }else if(!is.null(parInit$Psi))
-    {
+    }else if(!is.null(parInit$Psi)){
       k <- dim(parInit$Psi)[2]
-
-    }else if(!is.null(parInit$la))
-    {
+    }else if(!is.null(parInit$la)){
       k <- length(parInit$la)
     }
   }
 
-  if(!is.null(seed))
-  {
-   set.seed(seed)
-  }
+  if(!is.null(seed)) set.seed(seed)
 
   k <- sort(unique(k))
   nkv <- length(k)
-  if(length(index) !=2)
-  {
-    stop("id and time must be provided")
-  }
+  if(length(index) !=2) stop("id and time must be provided")
 
   id.which <- which(names(data) == index[1])
   tv.which <- which(names(data) == index[2])
 
-  if(length(id.which) == 0)
-  {
-    stop("the id column does not exist")
-  }
-
-  if(length(tv.which) == 0)
-  {
-    stop("the time column does not exist")
-  }
+  if(length(id.which) == 0) stop("the id column does not exist")
+  if(length(tv.which) == 0) stop("the time column does not exist")
 
   modSel <- match.arg(modSel, choices = eval(formals(lmest)$modSel))
   modManifest <- match.arg(modManifest, choices = eval(formals(lmest)$modManifest))
@@ -69,51 +48,41 @@ lmest <- function(responsesFormula = NULL, latentFormula = NULL,
   id <- data[,id.which]
   tv <- data[,tv.which]
 
-  if(is.character(id) | is.factor(id))
-  {
+  if(is.character(id) | is.factor(id)){
     #warning("id column must be numeric. Coerced in numeric.", call. = FALSE)
     id <- as.numeric(factor(id))
   }
 
-  if(is.character(tv) | is.factor(tv))
-  {
+  if(is.character(tv) | is.factor(tv)){
     #warning("time column must be numeric. Coerced in numeric.", call. = FALSE)
     tv <- as.numeric(factor(tv))
   }
 
-data.new <- data[,-c(id.which,tv.which), drop = FALSE]
+  data.new <- data[,-c(id.which,tv.which), drop = FALSE]
   ## of frequencies of the available configurations
-  if(is.null(responsesFormula))
-  {
+  if(is.null(responsesFormula)){
     Y <- data.new
     Xmanifest <- NULL
     Xinitial <- NULL
     Xtrans <- NULL
     Y_names <- colnames(Y)
     Xmanifest_names <- colnames(Xmanifest)
-    
   }else{
     temp <- getResponses(data = data.new,formula = responsesFormula)
     Y <- temp$Y
     Xmanifest <- temp$X
     Xinitial <- NULL
     Xtrans <- NULL
-
     Y_names <- colnames(Y)
     Xmanifest_names <- colnames(Xmanifest)
-
   }
 
-
-  if(!is.null(latentFormula))
-  {
+  if(!is.null(latentFormula)){
     temp <- getLatent(data = data.new,latent = latentFormula, responses = responsesFormula)
     Xinitial <- temp$Xinitial
     Xtrans <- temp$Xtrans
-
     Xinitial_names <- colnames(Xinitial)
     Xtrans_names <- colnames(Xtrans)
-
   }
   tmp <- long2matrices.internal(Y = Y, id = id, time = tv, yv = weights,
                                 Xinitial = Xinitial, Xmanifest = Xmanifest, Xtrans = Xtrans)
@@ -122,8 +91,7 @@ data.new <- data[,-c(id.which,tv.which), drop = FALSE]
   Xmanifest <- tmp$Xmanifest
   Xtrans <- tmp$Xtrans
   Y <- tmp$Y
-  if(is.null(weights))
-  {
+  if(is.null(weights)){
     freq = tmp$freq
   }else{
     freq = weights
@@ -133,11 +101,7 @@ data.new <- data[,-c(id.which,tv.which), drop = FALSE]
 
   miss = any(is.na(Y))
   if(miss){
-
-    if(model == "LMmanifest")
-    {
-      stop("Missing data in the dataset")
-    }
+    if(model == "LMmanifest") stop("Missing data in the dataset")
     cat("Missing data in the dataset, treated as missing at random\n")
     R = 1 * (!is.na(Y))
     Y[is.na(Y)] = 0
@@ -152,70 +116,59 @@ data.new <- data[,-c(id.which,tv.which), drop = FALSE]
     # df <- dim(Y)
     # Y <- apply(Y,2,function(x) x - min(x, na.rm = TRUE))
     # dim(Y) <- df
-    for(i in 1:dim(Y)[3])
-    {
+    for(i in 1:dim(Y)[3]){
       Y[,,i] <- Y[,,i]-min(Y[,,i],na.rm = TRUE)
     }
   }
 
   dimnames(Y)[[3]] <- Y_names
-
-  if(!is.null(Xmanifest))
-  {
-    if(any(is.na(Xmanifest)))
-    {
-      stop("missing data in the covariates affecting the measurement model are not allowed")
-    }
+  if(!is.null(Xmanifest)){
+    if(any(is.na(Xmanifest))) stop("missing data in the covariates affecting the measurement model are not allowed")
   }
-
-  if(!is.null(Xinitial))
-  {
-
+  if(!is.null(Xinitial)){
     dimnames(Xinitial)[[2]] <- Xinitial_names
-
-    if(any(is.na(Xinitial)))
-    {
+    if(any(is.na(Xinitial))){
       stop("missing data in the covariates affecting the initial probabilities are not allowed")
     }
   }
 
-
-  if(!is.null(Xtrans))
-  {
-
+  if(!is.null(Xtrans)){
     dimnames(Xtrans)[[3]] <- Xtrans_names
-
-    if(any(is.na(Xtrans)))
-    {
+    if(any(is.na(Xtrans))){
       stop("missing data in the covariates affecting the transition probabilities are not allowed")
     }
   }
 
-
   aicv = rep(NA,nkv)
   bicv = rep(NA,nkv)
   lkv = rep(NA,nkv)
 
   for(kv in 1:nkv){
     out[[kv]] <- switch(model,
-                       "LMbasic" = lmbasic(S = Y,yv = freq,k = k[kv],start = start,modBasic = modBasic,tol = tol,
-                                                maxit = maxit,out_se = out_se,piv = parInit$piv,Pi = parInit$Pi,Psi = parInit$Psi,miss = miss, R = R),
+                       "LMbasic" = lmbasic(S = Y,yv = freq,k = k[kv],start = start,
+                                           modBasic = modBasic,tol = tol,maxit = maxit,
+                                           out_se = out_se,piv = parInit$piv,Pi = parInit$Pi,
+                                           Psi = parInit$Psi,miss = miss, R = R,output=output,ntry = ntry),
                        "LMlatent" = lmcovlatent(S = Y,X1 = Xinitial,X2 = Xtrans,yv = freq,
-                                                      start = start,k = k[kv],tol = tol,maxit = maxit,paramLatent = paramLatent,output = output,
-                                                      Psi = parInit$Psi,Be = parInit$Be,Ga = parInit$Ga,fort = fort,out_se = out_se,fixPsi = ifelse(is.null(parInit$fixPsi),FALSE,parInit$fixPsi), miss = miss, R = R),
-                       "LMmanifest" = lmcovmanifest(S = as.matrix(Y[,,1]),X = Xmanifest, yv = freq, modManifest = modManifest,k = k[kv],tol = tol,maxit = maxit, q = q,
-                                                          start = start,out_se = out_se,mu = parInit$mu,al = parInit$al,be = parInit$be,si = parInit$si,rho = parInit$rho,la = parInit$la,PI = parInit$PI, output = output))
-
-
-
+                                                start = start,k = k[kv],tol = tol,maxit = maxit,
+                                                paramLatent = paramLatent,output = output,
+                                                Psi = parInit$Psi,Be = parInit$Be,Ga = parInit$Ga,
+                                                fort = fort,out_se = out_se,
+                                                fixPsi = ifelse(is.null(parInit$fixPsi),FALSE,parInit$fixPsi), 
+                                                miss = miss, R = R, ntry = ntry),
+                       "LMmanifest" = lmcovmanifest(S = as.matrix(Y[,,1]),X = Xmanifest, yv = freq,
+                                                    modManifest = modManifest,k = k[kv],tol = tol,
+                                                    maxit = maxit, q = q,start = start,
+                                                    out_se = out_se,mu = parInit$mu,al = parInit$al,
+                                                    be = parInit$be,si = parInit$si,
+                                                    rho = parInit$rho,la = parInit$la,PI = parInit$PI,
+                                                    output = output, ntry=ntry))
     aicv[kv] = out[[kv]]$aic
     bicv[kv] = out[[kv]]$bic
     lkv[kv] = out[[kv]]$lk
-
   }
 
-  if(modSel == "BIC")
-  {
+  if(modSel == "BIC") {
     best <- out[[which.min(bicv)]]
   }else if(modSel == "AIC"){
     best <- out[[which.min(aicv)]]
@@ -244,5 +197,4 @@ data.new <- data[,-c(id.which,tv.which), drop = FALSE]
 
   class(out) <- class(best)
   return(out)
-}
-
+}
\ No newline at end of file
